package com.usky.dxtop.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.dxtop.common.core.domain.model.LoginUser;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.*;
import com.usky.dxtop.common.utils.uuid.UUID;
import com.usky.dxtop.model.*;
import com.usky.dxtop.mapper.DreOrderMapper;
import com.usky.dxtop.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.service.api.WxApi;
import com.usky.dxtop.service.emun.AsyncResultType;
import com.usky.dxtop.service.emun.OrderPayType;
import com.usky.dxtop.service.emun.OrderSceneCode;
import com.usky.dxtop.service.emun.OrderStatus;
import com.usky.dxtop.service.vo.*;
import ma.glasnost.orika.MapperFacade;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yq
 * @since 2022-08-14
 */
@Service
public class DreOrderServiceImpl extends ServiceImpl<DreOrderMapper, DreOrder> implements DreOrderService {


    private static final String URL = "https://smartpark.caih.com/vuepay2/#/catering?id=";

    @Autowired
    private ISysAsyncTaskService sysAsyncTaskService;

    @Autowired
    private DreOrderDetailService dreOrderDetailService;

    @Autowired
    private SysFileService sysFileService;

    @Autowired
    private WxApi wxApi;

    @Autowired
    private DreUserService dreUserService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean add(DreOrder dreOrder) {
        String orderNumber = UUID.randomUUID().toString(true);
        dreOrder.setOrderNumber(orderNumber);
        this.save(dreOrder);
        dreOrderDetailService.insertList(dreOrder);
        ((DreOrderServiceImpl) AopContext.currentProxy()).sendWxMessage(dreOrder);
        return true;
    }

    @Override
    public boolean update(DreOrder dreOrder) {
        return this.updateById(dreOrder);
    }

    @Override
    public CommonPage<DreOrder> page(DreOrderRequest dreOrderRequest) {
        IPage<DreOrder> page = new Page<>(dreOrderRequest.getCurrent(), dreOrderRequest.getSize());
        LambdaQueryWrapper<DreOrder> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(StringUtils.isNotBlank(dreOrderRequest.getMType()),DreOrder::getMType,dreOrderRequest.getMType())
                .eq(null != dreOrderRequest.getOrderStatus(),DreOrder::getOrderStatus,dreOrderRequest.getOrderStatus())
                .eq(StringUtils.isNotBlank(dreOrderRequest.getOrderUserName()),DreOrder::getOrderUserName,dreOrderRequest.getOrderUserName())
                .eq(StringUtils.isNotBlank(dreOrderRequest.getPhone()),DreOrder::getPhone,dreOrderRequest.getPhone())
                .eq(StringUtils.isNotBlank(dreOrderRequest.getReceiveUserName()),DreOrder::getReceiveUserName,dreOrderRequest.getReceiveUserName())
                .eq(StringUtils.isNotBlank(dreOrderRequest.getSendUserName()),DreOrder::getSendUserName,dreOrderRequest.getSendUserName())
                .eq(null != dreOrderRequest.getOrderUserId() && 0 != dreOrderRequest.getOrderUserId(),DreOrder::getOrderUserId,dreOrderRequest.getOrderUserId())
                .eq(null != dreOrderRequest.getReceiveUserId() && 0 != dreOrderRequest.getReceiveUserId(),DreOrder::getReceiveUserId,dreOrderRequest.getReceiveUserId())
                .eq(null != dreOrderRequest.getSendUserId() && 0 != dreOrderRequest.getSendUserId(),DreOrder::getSendUserId,dreOrderRequest.getSendUserId())
                .in(CollectionUtils.isNotEmpty(dreOrderRequest.getStatusList()),DreOrder::getOrderStatus,dreOrderRequest.getStatusList())
                .orderByDesc(DreOrder::getId);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public String exportDetail(DreOrderRequest dreOrderRequest) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = loginUser.getUser();
        Long taskId = sysAsyncTaskService.getId(user.getUserName(), "订单详情");
        if (null == taskId) {
            throw new CustomException("获取次数太多了请稍后获取");
        }
        ((DreOrderServiceImpl) AopContext.currentProxy()).syncOrder(taskId,dreOrderRequest);
        return String.valueOf(taskId);
    }


    @Async
    public void syncOrder(Long taskId, DreOrderRequest dreOrderRequest){
        Workbook workbook = null;
        try {
            ExportParams params = new ExportParams(null, "订单详情");
            workbook = ExcelExportUtil.exportBigExcel(params, OrderExport.class,
                    (o, i) -> {
                        dreOrderRequest.setCurrent(i);
                        dreOrderRequest.setSize(30);
                        CommonPage<DreOrder> page = this.page(dreOrderRequest);
                        return page.getRecords().stream().map(this::orderChangeOrderExport).collect(Collectors.toList());
                    },null);
            if (null != workbook) {
                String fileName = String.format("%s.xls", "订单详情");
                MultipartFile multipartFile = ExcelUtils.downLoadExcel(workbook, fileName);
                uploadOrderFile(multipartFile,taskId);
            } else {
                sysAsyncTaskService.sendResult(taskId, false, "订单表格数据为空", AsyncResultType.TEXT);
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            sysAsyncTaskService.sendResult(taskId, false, "导出订单文件失败", AsyncResultType.TEXT);
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }

        }
    }


    public void uploadOrderFile(MultipartFile multipartFile, Long taskId){
        List<MultipartFile> list = new ArrayList<>();
        list.add(multipartFile);
        String batchNo = sysFileService.generateBatchNo();
        SysFile sysFile = new SysFile();
        sysFile.setBatchNo(batchNo);
        sysFile.setFiles(list);
        sysFile.setUploadType(SysFileServiceImpl.FILE);
        sysFile.setBusinessType("订单详情");
        sysFile.setExpriceAt(DateUtils.addDays(new Date(), 3));
        List<SysFile> sysFileList = sysFileService.addFile(sysFile);
        long count = sysFileList.stream().filter(sysFileDTO -> !sysFileDTO.getSuccess()).count();
        if (count > 0) {
            sysAsyncTaskService.sendResult(taskId, false, "上传文件失败", AsyncResultType.TEXT);
            return;
        }
        List<String> idList = sysFileList.stream().map(sf -> sf.getId().toString()).collect(Collectors.toList());
        boolean active = sysFileService.active(batchNo,idList);
        if (active) {
            sysAsyncTaskService.sendResult(taskId, true, batchNo, AsyncResultType.FILE);
        } else {
            sysAsyncTaskService.sendResult(taskId, false, "上传文件失败", AsyncResultType.TEXT);
        }
    }


    public DreOrderExport orderChangeOrderExport(DreOrder dreOrder){
        MapperFactory factory = new DefaultMapperFactory.Builder().build();
        factory.classMap(Order.class, OrderExport.class)
                .exclude("orderStatus")
                .byDefault().register();
        MapperFacade mapper = factory.getMapperFacade();
        DreOrderExport orderExport = mapper.map(dreOrder, DreOrderExport.class);
        orderExport.setOrderStatus(OrderStatus.parse(dreOrder.getOrderStatus()).getName());
        return orderExport;
    }


    @Async
    public void sendWxMessage(DreOrder dreOrder){
        String url = String.format("%s%s",URL,dreOrder.getId());
        List<DreUserVO> list = dreUserService.list(null, true);
        list = list.stream().filter(dreUserVO -> StringUtils.isNotBlank(dreUserVO.getOpenId())).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(list)){
            for (DreUserVO dreUserVO:list) {
                String result = wxApi.sendMessageApi(dreUserVO.getOpenId(),getMessageData(dreOrder),wxApi.getToken(WxApi.APP_ID,WxApi.SECRET), WxApi.DISH_TEMPLATE_ID,url);
                wxApi.againSendMessage(result,dreUserVO.getOpenId(), getMessageData(dreOrder), WxApi.DISH_TEMPLATE_ID, URL,WxApi.APP_ID,WxApi.SECRET);
            }
        }
    }

    public Map<String,Object> getMessageData(DreOrder dreOrder){
        Map<String, Object> map = new HashMap<>();
        map.put("first",getMap("您好，您有新订单",null));
        map.put("keyword1",getMap(dreOrder.getOrderNumber(),null));
        map.put("keyword2",getMap(dreOrder.getOrderUserName(),null));
        map.put("keyword3",getMap(DateUtils.format(dreOrder.getOrderTime(),null),null));
        map.put("keyword4",getMap(dreOrder.getAddress(),null));
        map.put("keyword5",getMap(dreOrder.getMType(),null));
        return map;
    }

    public Map<String,Object> getMap(Object value,String color){
        Map<String,Object> map = new HashMap<>();
        map.put("value",value);
        map.put("color",color);
        return map;
    }

}
