package com.usky.dxtop.service.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.dxtop.model.*;
import com.usky.dxtop.service.*;
import com.usky.dxtop.service.api.FpApi;
import com.usky.dxtop.service.api.OneCardApi;
import com.usky.dxtop.service.emun.ProductOrderStatus;
import com.usky.dxtop.service.impl.ProductOrderServiceImpl;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 卡消费
 * @author yq
 * @date 2021/9/16 9:31
 */
@Slf4j
@Component("cardJob")
public class CardDishJob {


    @Autowired
    private ProductOrderService productOrderService;

    @Autowired
    private CallApiLogService callApiLogService;


    public void execute(String param){
        AtomicReference<String> result = new AtomicReference<>("");
        List<CallApiLog> logs = new ArrayList<>();
        List<ProductOrder> data = getData(param);
        data.forEach(productOrder -> {
            TreeMap<String, String> treeMap = OneCardApi.cardPayBackParam(productOrder.getId());
            try {
                Map<String,String> map = new HashMap<>();
                map.put("Content-Type","application/x-www-form-urlencoded");
                result.set(OneCardApi.commonRequest(OneCardApi.CARD_PAYCHECK, OneCardApi.joinParam(treeMap), map, jsonObject -> {
                    productOrderService.checkCarPayResult(productOrder, jsonObject);
                    return null;
                }));
            }catch (Exception e){
                result.set(e.getMessage());
                log.error("雄伟消费记录查询失败，订单编号:"+productOrder.getOrderNumber()+"异常信息"+e);
            }
            logs.add(callApiLogService.getApiLog(productOrder.getOrderNumber(),
                    ProductOrderServiceImpl.FIND_ORDER_INFO,
                    OneCardApi.CARD_PAYCHECK,
                    JSONObject.toJSONString(treeMap),
                    result.get()));
        });
        productOrderService.updateBatchById(data);
        callApiLogService.saveBatch(logs);
        log.info("---cardDishJob---处理完成");
    }

    public List<ProductOrder> getData(String param){
        CardJobParam cardJobParam = JSON.parseObject(param, CardJobParam.class);
        LambdaQueryWrapper<ProductOrder> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(ProductOrder::getOrderFlag, ProductOrderStatus.NO_PAY.getCode());
        return productOrderService.list(queryWrapper);
    }


    @Data
    public static class CardJobParam {
        private String orderNumber;
    }
}
